/*******************************************************************************
 * Copyright 2020 Graz University of Technology
 * MOA ZS has been developed in a cooperation between EGIZ
 * and Graz University of Technology.
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
package at.gv.egiz.moazs.backend;

import at.gv.egiz.moazs.scheme.Marshaller;
import at.gv.egiz.moazs.scheme.MsgResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.util.concurrent.CompletableFuture;

import static java.util.concurrent.CompletableFuture.completedFuture;

/**
 * @author Christof Rabensteiner
 *
 */
@Component
public class LogResponseSink {

    private static final Logger log = LoggerFactory.getLogger(LogResponseSink.class);

    private final Marshaller msgMarshaller;

    public LogResponseSink(Marshaller msgMarshaller) {
        this.msgMarshaller = msgMarshaller;
    }

    /**
     * Log response to class logger at level=INFO.
     *
     * @param msgResponse
     * @return
     */
    public CompletableFuture<Void> log(MsgResponse msgResponse) {

        if(log.isInfoEnabled()) {
            var builder = new StringBuilder("Received the following Message: \n");
            builder.append(msgMarshaller.marshallXml(msgResponse.getResponseAsJAXBElement()));
            log.info(builder.toString());
        } else {
            log.warn("Will not log response because INFO level is disabled.");
        }

        return completedFuture(null);

    }
}
